#include <stdio.h>
#include <string.h>
#include <stdbool.h>
#include <stdlib.h>

#define ENTRY_INDEX "entries/entry-index"
#define PAGE_INDEX "page-index"
#define TEMPLATE "template.html"
#define BAR "bar.html"
#define WEBPAGE "index.html"
#define LINE_LEN 80
#define TITLE_OPEN "<div class=\"title4\">"
#define TITLE_CLOSE "</div>"
#define BIG_SPACE "<div class=\"space\"></div>"
#define SMALL_SPACE "<div class=\"small_space\"></div>"

void remove_newline(char *str)
{
	str[strcspn(str, "\n")] = 0;
}

void fprintf_smllspc(FILE *stream)
{
	fprintf(stream, "%s\n", SMALL_SPACE);
}

void fprintf_bigspc(FILE *stream)
{
	fprintf(stream, "%s\n", BIG_SPACE);
}

void fprint_date(FILE *stream, char *id)
{
	int mon = ((id[0] - '0') * 10) + id[1] - '0';
	int day = ((id[2] - '0') * 10) + id[3] - '0';
	int year = ((id[4] - '0') * 10) + id[5] - '0';

	switch(mon)
	{
		case 1:
			fprintf(stream, "January");
			break;
		case 2:
			fprintf(stream, "February");
			break;
		case 3:
			fprintf(stream, "March");
			break;
		case 4:
			fprintf(stream, "April");
			break;
		case 5:
			fprintf(stream, "May");
			break;
		case 6:
			fprintf(stream, "June");
			break;
		case 7:
			fprintf(stream, "July");
			break;
		case 8:
			fprintf(stream, "August");
			break;
		case 9:
			fprintf(stream, "September");
			break;
		case 10:
			fprintf(stream, "October");
			break;
		case 11:
			fprintf(stream, "November");
			break;
		case 12:
			fprintf(stream, "December");
			break;
	}
	
	fprintf(stream, " the %dth, 20%d:\n", day, year);	
}

int update_navbar()
{
	FILE *index = fopen(PAGE_INDEX, "r");
	FILE *navbar = fopen(BAR, "r");
	FILE *in;
	FILE *out;	

	char page_path[LINE_LEN];
	char line[LINE_LEN];
	char bar_line[LINE_LEN];
	bool mark_found;
	if(index == NULL)
	{
		printf("Error: could not locate '%s'\n", PAGE_INDEX);
		return 1;
	}
	if(navbar == NULL)
	{
		printf("Error: could not locate '%s'\n", BAR);
		return 1;
	}
	while(fscanf(index, "%s", page_path) == 1)
	{	
		rewind(navbar);
		mark_found = false;

		in = fopen(page_path, "r");
		out = fopen("temp.html", "w");

		if(in == NULL)
		{
			printf("Error: could not locate '%s'\n", page_path);
			return 1;
		}

		printf("Copying %s...\n", page_path);
		
		// copies in to out until it hits the bar mark	
		while(!mark_found)
		{
			fgets(line, LINE_LEN, in);
			fprintf(out, "%s", line);
			if(strcmp(line, "<!-- navbar begin -->\n") == 0)
			{
				mark_found = true;
			}
			if(feof(in) && !mark_found)
			{
				printf("Error: could not locate bar mark in '%s'\n", page_path);
				return 1;
			}
		}
		// copies navbar to out	
		while(fgets(bar_line, LINE_LEN, navbar) != NULL)
		{
			fprintf(out, "%s", bar_line);	
		}
		mark_found = false;
		// searches for the bar end mark
		while(!mark_found)
		{
			fgets(line, LINE_LEN, in);
			if(strcmp(line, "<!-- navbar end -->\n") == 0)
			{	
				mark_found = true;
			}
			if(feof(in) && !mark_found)
			{
				printf("Error: could not locate bar end mark in '%s'\n", page_path);
				return 1;
			}
		}
		// copies the rest of in to out
		fprintf(out, "<!-- navbar end -->\n");
		while(fgets(line, LINE_LEN, in) != NULL)
		{
			fprintf(out, "%s", line);	
		}

		fclose(out);	
		fclose(in);
		remove(page_path);
		rename("temp.html", page_path);
	}

	fclose(navbar);

	return 0;
}

int update_journal(FILE *index, FILE *template, FILE *page)
{
	rewind(index);
	rewind(template);
	rewind(page);

	bool mark_found = false;
	char temp_line[LINE_LEN];
	while(!mark_found)
	{
		fgets(temp_line, LINE_LEN, template);
		fprintf(page, "%s", temp_line);
		if(strcmp(temp_line, "<!-- journal begin -->\n") == 0)
		{
			mark_found = true;
		}
		if(feof(template) && !mark_found)
		{
			printf("Error: could not locate journal mark\n");
			return 1;
		}
	}

	char entry_path[13];
	char entry_id[8];
	char entry_title[LINE_LEN];
	char entry_line[LINE_LEN];
	FILE *entry;
	while(fscanf(index, "%s", entry_id) == 1)
	{
		strcpy(entry_path, "entries/");
		strcat(entry_id, "\0"); // adds string terminator
		strcat(entry_path, entry_id);
		entry = fopen(entry_path, "r");
		if(entry == NULL)
		{
			printf("Error: Could not locate '%s'\n", entry_id);
			return 1;
		}	
		printf("Copying %s...\n", entry_id);
		
		fprintf(page, "%s%s%s\n", "<a id=\"", entry_id, "\">");
		fgets(entry_title, LINE_LEN, entry);
		remove_newline(entry_title);
		fprintf(page, "%s%s%s\n", TITLE_OPEN, entry_title, TITLE_CLOSE);
		fprintf_smllspc(page);
		fprint_date(page, entry_id);
		fprintf_bigspc(page);
			
		while(fgets(entry_line, LINE_LEN, entry) != NULL)
		{	
			if(strncmp(entry_line, "#img=", 4) == 0)
			{
				strcpy(entry_line, entry_line + 5);
				remove_newline(entry_line);
				fprintf(page, "<center><a href=%s>", entry_line);
				fprintf(page, "<div class=\"img\"><img class=\"blog\" src=");
				fprintf(page, "%s>", entry_line);
				fprintf(page, "</div></a></center>\n");
			}	
			else if(strncmp(entry_line, "#cimg=", 5) == 0)
			{
				strcpy(entry_line, entry_line + 6);
				remove_newline(entry_line);
				fprintf(page, "<center><a href=%s>", entry_line);
				fprintf(page, "<div class=\"img\"><img class=\"blog\" src=");
				fprintf(page, "%s>", entry_line);
				fgets(entry_line, LINE_LEN, entry);
				fprintf(page, "</a>%s</div></center>\n", entry_line);
			}	
			else if(strncmp(entry_line, "#br", 2) == 0)
			{				
				fprintf(page, "<br>\n");
			}
			else if(strncmp(entry_line, "#ss", 2) == 0)
			{				
				fprintf_smllspc(page);
			}
			else if(strncmp(entry_line, "#bs\n", 2) == 0)
			{				
				fprintf_bigspc(page);
			}
			else
			{	
				fprintf(page, "%s", entry_line);
			}
		}
		fprintf(page, "%s\n", BIG_SPACE); fprintf(page, "%s%s%s\n", "<div class=\"title1\"><a href=\"", entry_path,
			   	"\">View raw</a></div>\n");
		fprintf(page, "%s\n", BIG_SPACE);	

		fclose(entry);	
	}

	while(fgets(temp_line, LINE_LEN, template) != NULL)
	{
		fprintf(page, "%s", temp_line);
	}

	return 0;	
}

int main()
{
	FILE *entry_index = fopen(ENTRY_INDEX, "r");
	FILE *template = fopen(TEMPLATE, "r");
	FILE *page = fopen(WEBPAGE, "w");

	bool error;

	if(entry_index == NULL)
	{
		printf("Error: Could not locate '%s'\n", ENTRY_INDEX);
		error = true;
	}
	if(page == NULL)
	{
		printf("Error: Could not locate '%s'\n", WEBPAGE);
		error = true;
	}

	if(!error)
	{
		if(!update_navbar())
		{
			printf("Navbar successfully updated\n");
		}
		else
		{
			printf("Could not update navbar\n");
		}
		if(!update_journal(entry_index, template, page))
		{
			printf("Journal successfully updated\n");
		}
		else
		{
			printf("Could not update journal\n");
		}
	}	

	fclose(entry_index);
	fclose(template);
	fclose(page);
}
